@@ -2,6 +2,7 @@
 import os
 import sys
 import logging
+from functools import wraps
 
 import numpy as np
 from vtk import (VTK_TETRA, VTK_QUADRATIC_TETRA, VTK_PYRAMID,
@@ -9,11 +10,12 @@ from vtk import (VTK_TETRA, VTK_QUADRATIC_TETRA, VTK_PYRAMID,
                  VTK_HEXAHEDRON, VTK_QUADRATIC_HEXAHEDRON, VTK_TRIANGLE, VTK_QUAD,
                  VTK_QUADRATIC_TRIANGLE, VTK_QUADRATIC_QUAD)
 import vtk
+import pyvista as pv
 
 from pyansys import _reader
 from pyansys.misc import vtk_cell_info
 from pyansys.geometry import Geometry
-from pyansys._cellqual import cell_quality_float, cell_quality
+from pyansys.cell_quality import quality
 
 VTK9 = vtk.vtkVersion().GetVTKMajorVersion() >= 9
 
@@ -27,6 +29,8 @@ class Archive(Geometry):
 
     Reads a blocked CDB file and optionally parses it to a vtk grid.
 
+    Write the archive file using ``CDWRITE, DB, archive.cdb``
+
     Parameters
     ----------
     filename : string
@@ -180,17 +184,12 @@ class Archive(Geometry):
         if self._grid is None:
             raise AttributeError('Archive must be parsed as a vtk grid.\n'
                                  'Set `parse_vtk=True`')
-        celltypes = self.grid.celltypes
-        points = self.grid.points
-        cells, offset = vtk_cell_info(self.grid)
-        if points.dtype == np.float64:
-            qual = cell_quality(cells, offset, celltypes, points)
-        else:
-            qual = cell_quality_float(cells, offset, celltypes, points)
+        return quality(self._grid)
 
-        # set qual of null cells to 1
-        qual[self._grid.celltypes == 0] = 1
-        return qual
+    @wraps(pv.plot)
+    def plot(self, *args, **kwargs):
+        """Plot the ANSYS archive file"""
+        self.grid.plot(*args, **kwargs)
 
 
 def chunks(l, n):
@@ -200,10 +199,11 @@ def chunks(l, n):
 
 
 def save_as_archive(filename, grid, mtype_start=1, etype_start=1,
-                    real_constant_start=1, mode='w',
-                    nblock=True, enum_start=1, nnum_start=1,
-                    include_etype_header=True,
-                    reset_etype=False, allow_missing=True):
+                    real_constant_start=1, mode='w', nblock=True,
+                    enum_start=1, nnum_start=1,
+                    include_etype_header=True, reset_etype=False,
+                    allow_missing=True, include_surface_elements=True,
+                    include_solid_elements=True):
     """Writes FEM as an ANSYS APDL archive file.  This function
     supports the following element types:
 
@@ -269,9 +269,42 @@ def save_as_archive(filename, grid, mtype_start=1, etype_start=1,
         determined by the shape of the element (i.e. quadradic
         tetrahedrals will be saved as SOLID187, linear hexahedrals as
         SOLID185).  Default True.
+
+    include_surface_elements : bool, optional
+        Includes surface elements when writing the archive file and
+        saves them as SHELL181.
+
+    include_solid_elements : bool, optional
+        Includes solid elements when writing the archive file and
+        saves them as SOLID185, SOLID186, or SOLID187.
+
     """
     header = '/PREP7\n'
 
+    if isinstance(grid, pv.PolyData):
+        grid = grid.cast_to_unstructured_grid()
+
+    allowable = []
+    if include_solid_elements:
+        allowable.extend([VTK_TETRA,
+                          VTK_QUADRATIC_TETRA,
+                          VTK_PYRAMID,
+                          VTK_QUADRATIC_PYRAMID,
+                          VTK_WEDGE,
+                          VTK_QUADRATIC_WEDGE,
+                          VTK_HEXAHEDRON,
+                          VTK_QUADRATIC_HEXAHEDRON])
+
+    if include_surface_elements:
+        allowable.extend([VTK_TRIANGLE,
+                          VTK_QUAD])
+                          # VTK_QUADRATIC_TRIANGLE,
+                          # VTK_QUADRATIC_QUAD
+
+    # extract allowable cell types
+    mask = np.in1d(grid.celltypes, allowable)
+    grid = grid.extract_cells(mask)
+
     # node numbers
     if 'ansys_node_num' in grid.point_arrays:
         nodenum = grid.point_arrays['ansys_node_num']
@@ -576,8 +609,15 @@ def save_as_archive(filename, grid, mtype_start=1, etype_start=1,
                               nodes[2])  # 3,  L (duplicate of K)
                 line += '%8d%8d%8d%8d\n' % writenodes
 
-            else:
-                raise RuntimeError('Invalid write cell type %d' % celltypes[i])
+            elif celltypes[i] == VTK_QUAD:
+                writenodes = (nodes[0],  # 0,  I
+                              nodes[1],  # 1,  J
+                              nodes[2],  # 2,  K
+                              nodes[3])  # 3,  L
+                line += '%8d%8d%8d%8d\n' % writenodes
+
+            # else:
+            #     raise RuntimeError('Invalid write cell type %d' % celltypes[i])
 
             f.write(line)
 
