@@ -1,12 +1,9 @@
 """Module to control interaction with an ANSYS shell instance.
-Built using ANSYS documentation from
-https://www.sharcnet.ca/Software/Ansys/
 
 This module makes no claim to own any rights to ANSYS.  It's merely an
 interface to software owned by ANSYS.
 
 """
-import time
 import glob
 import string
 import re
@@ -19,15 +16,20 @@ from shutil import copyfile, rmtree
 
 import appdirs
 import numpy as np
+import pyvista as pv
 
 import pyansys
 from pyansys.geometry_commands import geometry_commands
 from pyansys.element_commands import element_commands
 from pyansys.mapdl_functions import _MapdlCommands
 from pyansys.convert import is_float
+from pyansys import _reader
+
 
 MATPLOTLIB_LOADED = True
 try:
+    import matplotlib
+    matplotlib.use('tkagg')  # for windows
     import matplotlib.pyplot as plt
     import matplotlib.image as mpimg
 except:
@@ -183,7 +185,6 @@ def get_ansys_path(allow_input=True):
             exe_loc = f.read()
         # verify
         if not os.path.isfile(exe_loc) and allow_input:
-            print('Cached ANSYS executable %s not found' % exe_loc)
             exe_loc = save_ansys_path()
     elif allow_input:  # create configuration file
         exe_loc = save_ansys_path()
@@ -192,13 +193,27 @@ def get_ansys_path(allow_input=True):
 
 
 def change_default_ansys_path(exe_loc):
-    """
-    Change your default ansys path
+    """Change your default ansys path.
 
     Parameters
     ----------
     exe_loc : str
-        Ansys executable.  Must be a full path.
+        Ansys executable path.  Must be a full path.
+
+    Examples
+    --------
+    Change default ansys location on Linux
+
+    >>> import pyansys
+    >>> pyansys.change_default_ansys_path('/ansys_inc/v201/ansys/bin/ansys201')
+    >>> pyansys.mapdl.get_ansys_path()
+    '/ansys_inc/v201/ansys/bin/ansys201'
+
+    Change default ansys location on Windows
+    >>> ans_pth = 'C:/Program Files/ANSYS Inc/v193/ansys/bin/win64/ANSYS193.exe'
+    >>> pyansys.change_default_ansys_path(ans_pth)
+    >>> pyansys.mapdl.check_valid_ansys()
+    True
 
     """
     if os.path.isfile(exe_loc):
@@ -210,7 +225,10 @@ def change_default_ansys_path(exe_loc):
 
 def save_ansys_path(exe_loc=''):
     """ Find ANSYS path or query user """
-    print('Cached ANSYS executable %s not found' % exe_loc)
+    if exe_loc.strip():
+        print('Cached ANSYS executable %s not found' % exe_loc)
+    else:
+        print('Cached ANSYS executable not found')
     exe_loc, ver = find_ansys()
     if os.path.isfile(exe_loc):
         print('Found ANSYS at %s' % exe_loc)
@@ -536,7 +554,7 @@ def launch_mapdl(exec_file=None, run_location=None,
                                     'pyansys.Mapdl(exec_file=...)')
     else:  # verify ansys exists at this location
         if not os.path.isfile(exec_file):
-            raise FileNotFoundError('Invalid ANSYS executable at %s'
+            raise FileNotFoundError('Invalid ANSYS executable at "%s"\n'
                                     % exec_file + 'Enter one manually using '
                                     'pyansys.Mapdl(exec_file=)')
 
@@ -869,6 +887,8 @@ class _Mapdl(_MapdlCommands):
         self.response = None
         self._outfile = None
         self._show_matplotlib_figures = True
+        self._archive_cache = None
+        self._vtk_grid_cache = None
 
         self._log = setup_logger(loglevel.upper())
         self.non_interactive = self._non_interactive(self)
@@ -886,6 +906,11 @@ class _Mapdl(_MapdlCommands):
         if self._interactive_plotting:
             self.enable_interactive_plotting()
 
+    def _reset_cache(self):
+        """Reset cached NBLOCK and other items"""
+        self._archive_cache = None
+        self._grid_cache = None
+
     @property
     def _lockfile(self):
         """lockfile path"""
@@ -947,6 +972,187 @@ class _Mapdl(_MapdlCommands):
             self._apdl_log.close()
         self._apdl_log = None
 
+    def nplot(self, knum="", vtk=False, **kwargs):
+        """APDL Command: NPLOT
+
+        Displays nodes.
+
+        Parameters
+        ----------
+        knum : int, optional
+            Node number key:
+
+            - 0 : No node numbers on display (default).
+            - 1 : Include node numbers on display.  See also /PNUM command.
+
+        vtk : bool, optional
+           Display nodes using VTK.
+
+        Examples
+        --------
+        Plot using VTK while showing labels and changing the background
+
+        >>> mapdl.prep7()
+        >>> mapdl.n(1, 0, 0, 0)
+        >>> mapdl.n(11, 10, 0, 0)
+        >>> mapdl.fill(1, 11, 9)
+        >>> mapdl.nplot(vtk=True, knum=True, background='w', color='k',
+                        off_screen=True, show_bounds=True)
+
+        Plot without using VTK
+
+        >>> mapdl.enable_interactive_plotting()
+        >>> mapdl.prep7()
+        >>> mapdl.n(1, 0, 0, 0)
+        >>> mapdl.n(11, 10, 0, 0)
+        >>> mapdl.fill(1, 11, 9)
+        >>> mapdl.nplot()
+
+        Notes
+        -----
+        Only selected nodes [NSEL] are displayed.  Elements need not
+        be defined.  See the DSYS command for display coordinate
+        system.
+
+        This command is valid in any processor.
+        """
+        if vtk:
+            pl = pv.Plotter(off_screen=kwargs.pop('off_screen', None))
+            pl.add_points(self.nodes, color=kwargs.pop('color', 'w'))
+            pl.show_axes()
+            if 'background' in kwargs:
+                pl.set_background(kwargs['background'])
+            if knum:
+                pl.add_point_labels(self.nodes, self.nnum)
+
+            if 'cpos' in kwargs:
+                pl.camera_position = kwargs['cpos']
+
+            if 'show_bounds' in kwargs:
+                if kwargs['show_bounds']:
+                    pl.show_bounds()
+
+            return pl.show()
+        else:
+            return super().nplot(knum, **kwargs)
+
+    @property
+    def nodes(self):
+        """The coordinates of the active selected nodes as an array.
+
+        Examples
+        --------
+        >>> import pyansys
+        >>> mapdl = pyansys.launch_mapdl()
+        >>> mapdl.prep7()
+        >>> mapdl.n(1, 0, 0, 0)
+        >>> mapdl.n(10, 10, 0, 0)
+        >>> mapdl.fill(1, 10, 9)
+        >>> mapdl.nodes
+        array([[ 1.,  0.,  0.],
+               [ 2.,  0.,  0.],
+               [ 3.,  0.,  0.],
+               [ 4.,  0.,  0.],
+               [ 5.,  0.,  0.],
+               [ 6.,  0.,  0.],
+               [ 7.,  0.,  0.],
+               [ 8.,  0.,  0.],
+               [ 9.,  0.,  0.],
+               [10.,  0.,  0.]])
+        """
+        return self._archive.nodes
+
+    @property
+    def nnum(self):
+        """The node numbers of the selected nodes
+
+        Examples
+        --------
+        >>> import pyansys
+        >>> mapdl = pyansys.launch_mapdl()
+        >>> mapdl.prep7()
+        >>> mapdl.n(1, 0, 0, 0)
+        >>> mapdl.n(10, 10, 0, 0)
+        >>> mapdl.fill(1, 10, 9)
+        >>> mapdl.nnum
+        array([ 1,  2,  3,  4,  5,  6,  7,  8,  9, 10], dtype=int32)
+        """
+        return self._archive.nnum
+
+    @property
+    def n_node(self):
+        """Number of nodes currently selected
+
+        Examples
+        --------
+        >>> mapdl.n_node
+        665
+        """
+        return int(self.get(entity='NODE', item1='COUNT'))
+
+    @property
+    def _archive(self):
+        """Write entire archive to ASCII and read it in as a pyansys.Archive """
+        if self._archive_cache is None:
+            # temporarily disable log
+            prior_log_level = self._log.level
+            self._log.setLevel('CRITICAL')
+
+            # write database to an archive file
+            arch_filename = os.path.join(self.path, 'tmp.cdb')
+            self.cdwrite('db', arch_filename)
+            self._archive_cache = pyansys.Archive(arch_filename, parse_vtk=False)
+            self._log.setLevel(prior_log_level)
+        return self._archive_cache
+
+    @property
+    def _vtk_grid(self):
+        """Current vtk unstructured grid"""
+        if self._vtk_grid_cache is None:
+            quadgrid = self._archive._parse_vtk()
+            self._vtk_grid_cache = quadgrid.linear_copy()
+        return self._vtk_grid_cache
+
+    @property
+    def elements(self):
+        """List of elements containing raw ansys information.
+
+        Each element contains 10 items plus the nodes belonging to the
+        element.  The first 10 items are:
+
+        - FIELD 0 : material reference number
+        - FIELD 1 : element type number
+        - FIELD 2 : real constant reference number
+        - FIELD 3 : section number
+        - FIELD 4 : element coordinate system
+        - FIELD 5 : death flag (0 - alive, 1 - dead)
+        - FIELD 6 : solid model reference
+        - FIELD 7 : coded shape key
+        - FIELD 8 : element number
+        - FIELD 9 : base element number (applicable to reinforcing elements only)
+        - FIELDS 10 - 30 : The nodes belonging to the element in ANSYS numbering.
+
+        Examples
+        --------
+        >>> import pyansys
+        >>> mapdl = pyansys.launch_mapdl(override=True, loglevel='WARNING')
+        >>> mapdl.et(1, 188)
+        >>> mapdl.n(1, 0, 0, 0)
+        >>> mapdl.n(2, 1, 0, 0)
+        >>> mapdl.n(3, 2, 0, 0)
+        >>> mapdl.n(4, 3, 0, 0)
+        >>> mapdl.e(1, 2)
+        >>> mapdl.e(2, 3)
+        >>> mapdl.e(3, 4)
+        >>> mapdl.elements
+        [array([1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 2], dtype=int32),
+         array([1, 1, 1, 1, 0, 0, 0, 0, 2, 0, 2, 3], dtype=int32),
+         array([1, 1, 1, 1, 0, 0, 0, 0, 3, 0, 3, 4], dtype=int32),
+         array([1, 1, 1, 1, 0, 0, 0, 0, 4, 0, 4, 5], dtype=int32)]
+
+        """
+        return self._archive.elem
+
     def enable_interactive_plotting(self, pixel_res=1600):
         """Enables interactive plotting.  Requires matplotlib
 
@@ -960,7 +1166,7 @@ class _Mapdl(_MapdlCommands):
         """
         if MATPLOTLIB_LOADED:
             self.show('PNG')
-            self.gfile(1200)
+            self.gfile(pixel_res)
             self._interactive_plotting = True
         else:
             raise ImportError('Install matplotlib to use enable interactive plotting\n' +
@@ -1060,6 +1266,51 @@ class _Mapdl(_MapdlCommands):
         else:
             raise Exception('Cannot run:\n%s\n' % command + 'File does not exist')
 
+    def eplot(self, vtk=False, **kwargs):
+        """APDL Command: EPLOT
+
+        Produces an element display.
+
+        Parameters
+        ----------
+        vtk : bool, optional
+            Plot the currently selected elements using ``pyvista``.
+
+        **kwargs
+            See ``help(pyvista.plot)`` for more keyword arguments
+            related to visualizing using ``vtk``.
+
+        Notes
+        -----
+        Produces an element display of the selected elements. In full
+        graphics, only those elements faces with all of their
+        corresponding nodes selected are plotted. In PowerGraphics,
+        all element faces of the selected element set are plotted
+        irrespective of the nodes selected.  However, for both full
+        graphics and PowerGraphics, adjacent or otherwise duplicated
+        faces of 3-D solid elements will not be displayed in an
+        attempt to eliminate plotting of interior facets. See the DSYS
+        command for display coordinate system issues.
+
+        This command will display curvature in midside node elements
+        when PowerGraphics is activated [/GRAPHICS,POWER] and
+        /EFACET,2 or /EFACET,4 are enabled.  (To display curvature,
+        two facets per edge is recommended [/EFACET,2]).  When you
+        specify /EFACET,1, PowerGraphics does not display midside
+        nodes. /EFACET has no effect on EPLOT for non-midside node
+        elements.
+
+        This command is valid in any processor.
+        """
+        if vtk:
+            # default kwargs
+            kwargs.setdefault('color', 'w')
+            kwargs.setdefault('show_axes', True)
+            kwargs.setdefault('show_edges', True)
+            return self._vtk_grid.plot(**kwargs)
+        else:
+            return super().eplot(**kwargs)
+
     @property
     def processor(self):
         """ Returns the current processor """
@@ -1097,7 +1348,7 @@ class _Mapdl(_MapdlCommands):
         """
         # load ansys parameters to python
         filename = os.path.join(self.path, 'parameters.parm')
-        self.Parsav('all', filename)
+        self.parsav('all', filename)
         self.parameters, self.arrays = load_parameters(filename)
         return self.parameters, self.arrays
 
@@ -1171,10 +1422,17 @@ class _Mapdl(_MapdlCommands):
         """Returns a binary interface to the result file."""
         try:
             result_path = self.inquire('RSTFILE')
-            if not os.path.dirname(result_path):
-                result_path = os.path.join(self.path, '%s.rst' % result_path)
-        except:
+        except RuntimeError:
+            result_path = ''
+
+        if not result_path:
             result_path = os.path.join(self.path, '%s.rst' % self._jobname)
+        elif not os.path.dirname(result_path):
+            result_path = os.path.join(self.path, '%s.rst' % result_path)
+
+        # there may be multiple result files at this location (if not
+        # combining results)
+        
 
         if not os.path.isfile(result_path):
             raise FileNotFoundError('No results found at %s' % result_path)
@@ -1242,19 +1500,172 @@ class _Mapdl(_MapdlCommands):
 
     def get_float(self, entity="", entnum="", item1="", it1num="",
                   item2="", it2num="", **kwargs):
-        """Get the value of a float-parameter from APDL.
+        """Runs the *GET command with a default parameter
+
+        See `help(get)` for more details.
+
+        Parameters
+        ----------
+        entity
+            Entity keyword. Valid keywords are NODE, ELEM, KP, LINE, AREA,
+            VOLU, PDS, etc., as shown for Entity = in the tables below.
+
+        entnum
+            The number or label for the entity (as shown for ENTNUM = in the
+            tables below). In some cases, a zero (or blank) ENTNUM represents
+            all entities of the set.
+
+        item1
+            The name of a particular item for the given entity. Valid items are
+            as shown in the Item1 columns of the tables below.
+
+        it1num
+            The number (or label) for the specified Item1 (if any). Valid
+            IT1NUM values are as shown in the IT1NUM columns of the tables
+            below. Some Item1 labels do not require an IT1NUM value.
+
+        item2, it2num
+            A second set of item labels and numbers to further qualify the item
+            for which data are to be retrieved. Most items do not require this
+            level of information.
+
+        Returns
+        -------
+        par : float
+            Floating point value of the parameter.
+
+        Examples
+        --------
+        Retreive the number of nodes.
+
+        >>> value = ansys.get('node', '', 'count')
+        >>> value
+        3003
+
+        Retreive the number of nodes using keywords.
+
+        >>> value = ansys.get(entity='node', item1='count')
+        >>> value
+        3003
+        """
+        return self.get(entity=entity, entnum=entnum, item1=item1, it1num=it1num,
+                        item2=item2, it2num=it2num, **kwargs)
+
+    def get(self, par="__floatparameter__", entity="", entnum="",
+            item1="", it1num="", item2="", it2num="", **kwargs):
+        """APDL Command: *GET
+
+        Retrieves a value and stores it as a scalar parameter or part
+        of an array parameter.
+
+        Parameters
+        ----------
+        par
+            The name of the resulting parameter. See *SET for name
+            restrictions.
+
+        entity
+            Entity keyword. Valid keywords are NODE, ELEM, KP, LINE, AREA,
+            VOLU, PDS, etc., as shown for Entity = in the tables below.
+
+        entnum
+            The number or label for the entity (as shown for ENTNUM = in the
+            tables below). In some cases, a zero (or blank) ENTNUM represents
+            all entities of the set.
+
+        item1
+            The name of a particular item for the given entity. Valid items are
+            as shown in the Item1 columns of the tables below.
+
+        it1num
+            The number (or label) for the specified Item1 (if any). Valid
+            IT1NUM values are as shown in the IT1NUM columns of the tables
+            below. Some Item1 labels do not require an IT1NUM value.
+
+        item2, it2num
+            A second set of item labels and numbers to further qualify the item
+            for which data are to be retrieved. Most items do not require this
+            level of information.
+
+        Returns
+        -------
+        par : float
+            Floating point value of the parameter.
+
+        Examples
+        --------
+        Retreive the number of nodes
+
+        >>> value = ansys.get('val', 'node', '', 'count')
+        >>> value
+        3003
+
+        Retreive the number of nodes using keywords.  Note that the
+        parameter name is optional.
+
+        >>> value = ansys.get(entity='node', item1='count')
+        >>> value
+        3003
 
         Notes
         -----
-        An APDL parameter __floatparameter__ is created/overwritten.
+        *GET retrieves a value for a specified item and stores the value as a
+        scalar parameter, or as a value in a user-named array parameter. An
+        item is identified by various keyword, label, and number combinations.
+        Usage is similar to the *SET command except that the parameter values
+        are retrieved from previously input or calculated results. For example,
+        *GET,A,ELEM,5,CENT,X returns the centroid x-location of element 5 and
+        stores the result as parameter A. *GET command operations, along with
+        the associated Get functions return values in the active coordinate
+        system unless stated otherwise. A Get function is an alternative in-
+        line function that can be used to retrieve a value instead of the *GET
+        command (see Using In-line Get Functions for more information).
+
+        Both *GET and *VGET retrieve information from the active data stored in
+        memory. The database is often the source, and sometimes the information
+        is retrieved from common memory blocks that the program uses to
+        manipulate information. Although POST1 and POST26 operations use a
+        *.rst file, *GET data is accessed from the database or from the common
+        blocks. Get operations do not access the *.rst file directly. For
+        repeated gets of sequential items, such as from a series of elements,
+        see the *VGET command.
+
+        Most items are stored in the database after they are calculated and are
+        available anytime thereafter. Items are grouped according to where they
+        are usually first defined or calculated. Preprocessing data will often
+        not reflect the calculated values generated from section data. Do not
+        use *GET to obtain data from elements that use calculated section data,
+        such as beams or shells. Most of the general items listed below are
+        available from all modules. Each of the sections for accessing *GET
+        parameters are shown in the following order:
+
+        *GET General Entity Items
+
+        *GET Preprocessing Entity Items
+
+        *GET Solution Entity Items
+
+        *GET Postprocessing Entity Items
+
+        *GET Probabilistic Design Entity Items
+
+        The *GET command is valid in any processor.
         """
-        line = self.get("__floatparameter__", entity, entnum, item1, it1num,
-                        item2, it2num, **kwargs)
-        return float(re.search(r"(?<=VALUE\=).*", line).group(0))
+        command = "*GET,%s,%s,%s,%s,%s,%s,%s" % (str(par),
+                                                 str(entity),
+                                                 str(entnum),
+                                                 str(item1),
+                                                 str(it1num),
+                                                 str(item2),
+                                                 str(it2num))
+        response = self.run(command, **kwargs)
+        try:
+            return float(response.split('=')[-1])
+        except:
+            return None
 
     def read_float_parameter(self, parameter_name):
         """Read out the value of a ANSYS parameter to use in python.
-        Can raise TypeError.
 
         Parameters
         ----------
@@ -1265,13 +1676,38 @@ class _Mapdl(_MapdlCommands):
         -------
         float
             Value of ANSYS parameter.
+
+        Examples
+        --------
+        >>> ansys.get('myparm', 'node', '', 'count')
+        >>> value = ansys.read_float_parameter('myparm')
+        >>> value
+        3003
+
+        Retreive the value in an array.  This example creates a block
+        of elements and stores the element numbers in an array.  Then
+        it retreives the first value of that array
+
+        >>> ansys.prep7()
+        >>> ansys.block(0,1,0,1,0,1)
+        >>> ansys.et(1, 186)
+        >>> ansys.vmesh('all')
+        >>> ansys.run('*VGET, ELEM_ARR, ELEM, , elist')
+        >>> ansys.read_float_parameter('ELEM_ARR(1)')
+        1.0
+
+        You could also retrieve ELEM_ARR with
+
+        >>> values, arrays = ansys.load_parameters()
+        >>> arrays
+        {'ELEM_ARR': array([1., 2., 3., 4., 5., 6., 7., 8.])}
+
         """
         try:
-            line = self.run(parameter_name + " = " + parameter_name)
+            response = self.run(parameter_name + " = " + parameter_name)
         except TypeError:
-            print('Input variable parameter_name should be string')
-            raise
-        return float(re.search(r"(?<=\=).*", line).group(0))
+            raise TypeError('Input variable `parameter_name` should be string')
+        return float(response.split('=')[-1])
 
     def read_float_from_inline_function(self, function_str):
         """Use a APDL inline function to get a float value from ANSYS.
@@ -1426,6 +1862,255 @@ class _Mapdl(_MapdlCommands):
         raise NotImplementedError('\nCommand "Run" decpreciated.  \n'
                                   'Please use "run" instead')
 
+    def modal_analysis(self, method='lanb', nmode='', freqb='', freqe='', cpxmod='',
+                       nrmkey='', modtype='', memory_option='', elcalc=False):
+        """Run a modal with basic settings analysis
+
+        Parameters
+        ----------
+        method : str
+            Mode-extraction method to be used for the modal analysis.
+            Defaults to lanb (block lanczos).  Must be one of the following:
+
+            - LANB : Block Lanczos
+            - LANPCG : PCG Lanczos
+            - SNODE : Supernode modal solver
+            - SUBSP : Subspace algorithm
+            - UNSYM : Unsymmetric matrix
+            - DAMP : Damped system
+            - QRDAMP : Damped system using QR algorithm
+            - VT : Variational Technology
+
+        nmode : int, optional
+            The number of modes to extract. The value can depend on
+            the value supplied for Method. NMODE has no default and
+            must be specified. If Method = LANB, LANPCG, or SNODE, the
+            number of modes that can be extracted can equal the DOFs
+            in the model after the application of all boundary
+            conditions.
+
+        freqb : float, optional
+            The beginning, or lower end, of the frequency range of
+            interest.
+    
+        freqe : float, optional
+            The ending, or upper end, of the frequency range of
+            interest (in Hz). The default for Method = SNODE is
+            described below. The default for all other methods is to
+            calculate all modes, regardless of their maximum
+            frequency.
+
+        cpxmod : str, optional
+            Complex eigenmode key. Valid only when ``method='QRDAMP'``
+            or ``method='unsym'``
+
+            - AUTO : Determine automatically if the eigensolutions are
+              real or complex and output them accordingly. This is
+              the default for ``method='UNSYM'``.  Not supported for
+              Method = QRDAMP.
+            - ON or CPLX : Calculate and output complex eigenmode
+              shapes.
+            - OFF or REAL : Do not calculate complex eigenmode
+              shapes. This is required if a mode-
+              superposition analysis is intended after the
+              modal analysis for Method = QRDAMP. This is the
+              default for this method.
+
+        nrmkey : bool, optional
+            Mode shape normalization key.  When ``True`` (default),
+            normalize the mode shapes to the mass matrix.  When False,
+            Normalize the mode shapes to unity instead of to the mass
+            matrix.  If a subsequent spectrum or mode-superposition
+            analysis is planned, the mode shapes should be normalized
+            to the mass matrix.
+
+        modtype : str, optional
+            Type of modes calculated by the eigensolver. Only
+            applicable to the unsymmetric eigensolver.
+
+            - Blank : Right eigenmodes. This value is the default.
+            - BOTH : Right and left eigenmodes. The left eigenmodes are
+              written to Jobname.LMODE.  This option must be
+              activated if a mode-superposition analysis is intended.
+
+        memory_option : str, optional
+            Memory allocation option:
+
+            * ``DEFAULT`` - Default Memory mode
+                      Use the default memory allocation strategy for
+                      the sparse solver. The default strategy attempts
+                      to run in the INCORE memory mode. If there is
+                      not enough available physical memory when the
+                      solver starts to run in the ``INCORE`` memory
+                      mode, the solver will then attempt to run in the
+                      ``OUTOFCORE`` memory mode.
+
+            * ``INCORE`` - In-core memory mode
+                     Use a memory allocation strategy in the sparse
+                     solver that will attempt to obtain enough memory
+                     to run with the entire factorized matrix in
+                     memory. This option uses the most amount of
+                     memory and should avoid doing any I/O. By
+                     avoiding I/O, this option achieves optimal solver
+                     performance. However, a significant amount of
+                     memory is required to run in this mode, and it is
+                     only recommended on machines with a large amount
+                     of memory. If the allocation for in-core memory
+                     fails, the solver will automatically revert to
+                     out-of-core memory mode.
+
+            * ``OUTOFCORE`` - Out of core memory mode.
+                        Use a memory allocation strategy in the sparse
+                        solver that will attempt to allocate only
+                        enough work space to factor each individual
+                        frontal matrix in memory, but will store the
+                        entire factorized matrix on disk. Typically,
+                        this memory mode results in poor performance
+                        due to the potential bottleneck caused by the
+                        I/O to the various files written by the
+                        solver.
+
+        elcalc : bool, optional
+            Calculate element results, reaction forces, energies, and
+            the nodal degree of freedom solution.  Default ``False``.
+
+        Examples
+        --------
+        Modal analysis using default parameters for the first 6 modes
+
+        >>> mapdl.modal_analysis(nmode=6)
+
+        Notes
+        -----
+        For models that involve a non-symmetric element stiffness
+        matrix, as in the case of a contact element with frictional
+        contact, the QRDAMP eigensolver (MODOPT, QRDAMP) extracts
+        modes in the modal subspace formed by the eigenmodes from the
+        symmetrized eigenproblem. The QRDAMP eigensolver symmetrizes
+        the element stiffness matrix on the first pass of the
+        eigensolution, and in the second pass, eigenmodes are
+        extracted in the modal subspace of the first eigensolution
+        pass. For such non- symmetric eigenproblems, you should verify
+        the eigenvalue and eigenmode results using the non-symmetric
+        matrix eigensolver (MODOPT,UNSYM).
+
+        The DAMP and QRDAMP options cannot be followed by a subsequent
+        spectrum analysis. The UNSYM method supports spectrum analysis
+        when eigensolutions are real.
+
+        """
+        if nrmkey:
+            if nrmkey.lower() != 'off':
+                nrmkey = 'ON'
+        nrmkey = 'OFF'
+
+        self.slashsolu()
+        self.antype(2, 'new')
+        self.modopt(method, nmode, freqb, freqe, cpxmod, nrmkey, modtype)
+        self.bcsoption(memory_option)
+
+        if elcalc:
+            self.mxpand(elcalc='YES')
+
+        self.solve()
+        self.finish()
+
+    def __str__(self):
+        try:
+            stats = self.slashstatus()
+        except:
+            return 'MAPDL exited'
+
+        st = stats.find('*** YOU ARE IN')
+        en = stats.find('INITIAL', st)
+
+        version_str = '\n'.join(stats[st:en].splitlines()[1:])
+        version_str = version_str.split('CUSTOMER')[0].strip() + '\n\n'
+
+        st = stats.find('Current routine')
+        en = stats.find('Active options for this analysis type')
+
+        routine_str = ' ' + stats[st:en].strip()
+
+        pyansys_version_str = '\n\n pyansys version: %s' % pyansys.__version__
+        return version_str + routine_str + pyansys_version_str
+
+    def load_array(self, arr, name):
+        """Load a numpy array or python list directly to MAPDL
+
+        Writes the numpy array to disk and then reads it in within MAPDL
+        using *VREAD.
+
+        Parameters
+        ----------
+        arr : np.ndarray or List
+
+        name : str
+            Name of the array to write to within MAPDL.
+
+        Examples
+        --------
+        Load a 1D numpy array into MAPDL
+
+        >>> arr = np.array([10, 20, 30])
+        >>> mapdl.load_array(arr, 'MYARR')
+        >>> parm, mapdl_arrays = mapdl.load_parameters()
+        >>> mapdl_arrays['MYARR']
+        array([10., 20., 30.])
+
+        Load a 2D numpy array into MAPDL
+
+        >>> arr = np.random.random((5, 3))
+        >>> mapdl.load_array(arr, 'MYARR')
+        >>> parm, mapdl_arrays = mapdl.load_parameters()
+        >>> mapdl_arrays['MYARR']
+        array([[0.39806635, 0.15060953, 0.3990557 ],
+               [0.26837768, 0.02033222, 0.15655861],
+               [0.46110226, 0.06381489, 0.20068533],
+               [0.20122863, 0.5727896 , 0.85636037],
+               [0.68126612, 0.67460878, 0.3678797 ]])
+
+        Load a python list into MAPDL
+
+        >>> mapdl.load_array([10, -1, 8, 4, 10], 'MYARR')
+        >>> parm, mapdl_arrays = mapdl.load_parameters()
+        >>> mapdl_arrays['MYARR']
+        array([10., -1.,  8.,  4., 10.])
+
+        """
+        # type checks
+        arr = np.array(arr)
+        if not np.issubdtype(arr.dtype, np.number):
+            raise TypeError('Only numerical arrays or lists are supported')
+        if arr.ndim > 3:
+            raise ValueError('MAPDL VREAD only supports a arrays with a'
+                             ' maximum of 3 dimensions.')
+
+        name = name.upper()
+        # disable logging for this function
+        prior_log_level = self._log.level
+        self._log.setLevel('CRITICAL')
+
+        idim, jdim, kdim = arr.shape[0], 0, 0
+        if arr.ndim >= 2:
+            jdim = arr.shape[1]
+        if arr.ndim == 3:
+            kdim = arr.shape[2]
+
+        # write array from numpy to disk:
+        filename = os.path.join(self.path, '_tmp.dat')
+        if arr.dtype != np.double:
+            arr = arr.astype(np.double)
+        pyansys._reader.write_array(filename.encode(), arr.ravel('F'))
+
+        self.dim(name, imax=idim, jmax=jdim, kmax=kdim)
+        with self.non_interactive:
+            self.vread('%s(1, 1),%s,,,IJK, %d, %d, %d' % (name, filename,
+                                                          idim, jdim, kdim))
+            self.run('(1F20.12)')
+
+        self._log.setLevel(prior_log_level)
+
 
 # TODO: Speed this up with:
 # https://tinodidriksen.com/2011/05/cpp-convert-string-to-double-speed/
@@ -1523,4 +2208,6 @@ def load_parameters(filename):
                 append_varname = split_line[1].strip()
                 append_mode = True
 
+    for array_name in arrays:
+        arrays[array_name] = np.squeeze(arrays[array_name])
     return parameters, arrays
